import com.itbaizhan.mapper.UserMapper;
import com.itbaizhan.pojo.User;
import com.itbaizhan.pojo.PageQuery;
import org.apache.ibatis.io.Resources;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.io.InputStream;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/***
 * 利用Junit前置后置方法  优化测试类代码
 *
 * 执行test方法前都需要执行before   执行test方法后都需要执行after
 */
public class TestUserMapper2 {
    InputStream is = null;
    SqlSession session = null;
    UserMapper userMapper = null;
    @Before
    public void before() throws Exception {
        //读取核心配置文件
        is = Resources.getResourceAsStream("SqlMapConfig.xml");

        //创建SqlSessionFactoryBuilder对象
        SqlSessionFactoryBuilder builder = new SqlSessionFactoryBuilder();

        //SqlSessionFactoryBuilder对象获取sqlSessionFactory对象
        SqlSessionFactory factory = builder.build(is);

        //sqlSessionFactory对象获取SqlSession对象
        session = factory.openSession();

        //SqlSession对象获取代理对象
        userMapper = session.getMapper(UserMapper.class);
    }

    @After
    public void after() throws Exception {
        session.commit();
        session.close();
        is.close();
    }

    @Test
    public void testUpdate(){
        User user = new User(7, "xry", "女", "广州");
        userMapper.update(user);
    }

    @Test
    public void testDelete(){
        userMapper.delete(7);
    }

    @Test
    public void findByIdTest(){
        User byId = userMapper.findById(1);
        System.out.println(byId);
    }

    @Test
    public void findByUsernameLikeTest(){
//        List<User> users = userMapper.findByUsernameLike("%尚学堂%");
        List<User> users = userMapper.findByUsernameLike("尚学堂");
        users.forEach(System.out::println);
    }

    @Test
    public void testFindPage1(){
        List<User> users = userMapper.findPage1(0, 3);
        users.forEach(System.out::println);
    }

    @Test
    public void testFindPage2(){
        PageQuery pageQuery = new PageQuery(0, 3);
        List<User> users = userMapper.findPage2(pageQuery);
        users.forEach(System.out::println);
    }

    @Test
    public void testFindPage3(){
        Map params = new HashMap();
        params.put("startIndex", 0);
        params.put("pageSize", 4);
        List<User> page3 = userMapper.findPage3(params);
        page3.forEach(System.out::println);
    }

    @Test
    public void testFindCount(){
        int count = userMapper.findCount();
        System.out.println(count);
    }

    @Test
    public void testAdd2(){
        User user = new User("hhaha", "男", "beijing");
        userMapper.add2(user);
        session.commit();
        System.out.println(user);
    }

    @Test
    public void testFindByCondition(){
        User user = new User();
        List<User> users1 = userMapper.findByCondition(user);
        users1.forEach(System.out::println);

        user.setUsername("%尚学堂%");
        List<User> users2 = userMapper.findByCondition(user);
        users2.forEach(System.out::println);

        user.setAddress("北京");
        List<User> users3 = userMapper.findByCondition(user);
        users3.forEach(System.out::println);
    }

    @Test
    public void testUpdateUser(){
        User user = new User();
        user.setId(8);
        user.setUsername("我是程序猿");
        user.setSex("女");
        user.setAddress("巴中");
        userMapper.updateUser(user);
    }

    @Test
    public void testFindByUsername(){
        List<User> users1 = userMapper.findByUsername("北京hahahhhhhhhhh");
        users1.forEach(System.out::println);
    }

    @Test
    public void testDeleteBatch(){
        int[] ids = {8,9};
        userMapper.deleteBatch(ids);
    }

    @Test
    public void testInsertBatch(){
        User user1 = new User("程序员1", "男", "北京");
        User user2 = new User("程序员2", "女", "上海");
        List<User> users = new ArrayList();
        users.add(user1);
        users.add(user2);
        userMapper.insertBatch(users);

    }

    @Test
    public void testFindUser(){
        Map<String,Object> queryMap = new HashMap();
        queryMap.put("sex", "男");
        queryMap.put("address", "北京");
        List<User> user = userMapper.findUser(queryMap);
        user.forEach(System.out::println);

    }
}
