import com.itbaizhan.mapper.UserMapper;
import com.itbaizhan.pojo.User;
import org.apache.ibatis.io.Resources;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import org.junit.Test;

import java.io.IOException;
import java.io.InputStream;

public class TestUserMapper3 {
    @Test
    public void testCache1() throws IOException {
//        (1)读取核心配置文件
        InputStream is = Resources.getResourceAsStream("SqlMapConfig.xml");

//        (2)创建SqlSessionFactoryBuilder对象
        SqlSessionFactoryBuilder builder = new SqlSessionFactoryBuilder();

//        (3)SqlSessionFactoryBuilder对象获取sqlSessionFactory对象
        SqlSessionFactory factory = builder.build(is);

//        (4)sqlSessionFactory对象获取SqlSession对象
        SqlSession session = factory.openSession();

        UserMapper mapper1 = session.getMapper(UserMapper.class);
        UserMapper mapper2 = session.getMapper(UserMapper.class);

        User user1 = mapper1.findById(1);
        System.out.println(user1);
        System.out.println("======");
        /**
         * 1、SqlSession 调用 close() ：操作后SqlSession对象不可用，该对象的缓存数据也不可用。
         * 2、SqlSession 调用 clearCache() / commit() ：操作会清空一级缓存数据。
         * 3、SqlSession 调用增删改方法：操作会清空一级缓存数据，
         *                          因为增删改后数据库发生改变，缓存数据将不准确。
         */
        User user2 = mapper1.findById(1);
        System.out.println(user2);

    }


    @Test
    public void testCache2() throws IOException {
//        (1)读取核心配置文件
        InputStream is = Resources.getResourceAsStream("SqlMapConfig.xml");

//        (2)创建SqlSessionFactoryBuilder对象
        SqlSessionFactoryBuilder builder = new SqlSessionFactoryBuilder();

//        (3)SqlSessionFactoryBuilder对象获取sqlSessionFactory对象
        SqlSessionFactory factory = builder.build(is);

//        (4)sqlSessionFactory对象获取SqlSession对象
        SqlSession session1 = factory.openSession();
        SqlSession session2 = factory.openSession();

        UserMapper mapper1 = session1.getMapper(UserMapper.class);
        UserMapper mapper2 = session2.getMapper(UserMapper.class);

        User user1 = mapper1.findById(1);
        System.out.println(user1.hashCode());
        System.out.println("======");


        User user2 = mapper2.findById(1);
        System.out.println(user2.hashCode());
    }


    /***
     * 开启二级缓存：
     1、POJO类实现Serializable接口。

     2、在MyBatis配置文件添加如下设置：
     <settings>
        <setting name="cacheEnabled"
        value="true"/>
     </settings>
     由于cacheEnabled默认值是true，所以该设置可以省略。

     3、在映射文件添加 <cache /> 标签，该映射文件下的所有方法都支持二级缓存。

     4、如果查询到的集合中对象过多，二级缓存只能缓存1024个对象引用。
     可以通过 <cache /> 标签的size属性修改该数量。
     <cache size="2048"/>

     */


    @Test
    public void testCache3() throws IOException {
//        (1)读取核心配置文件
        InputStream is = Resources.getResourceAsStream("SqlMapConfig.xml");

//        (2)创建SqlSessionFactoryBuilder对象
        SqlSessionFactoryBuilder builder = new SqlSessionFactoryBuilder();

//        (3)SqlSessionFactoryBuilder对象获取sqlSessionFactory对象
        SqlSessionFactory factory = builder.build(is);

//        (4)sqlSessionFactory对象获取SqlSession对象
        SqlSession session1 = factory.openSession();
        SqlSession session2 = factory.openSession();

        UserMapper mapper1 = session1.getMapper(UserMapper.class);
        UserMapper mapper2 = session2.getMapper(UserMapper.class);

        User user1 = mapper1.findById(1);
        System.out.println(user1);
        System.out.println(user1.hashCode());
        System.out.println("======");
        //让一级缓存失效
        session1.commit();

        User user2 = mapper2.findById(1);
        System.out.println(user2);
        System.out.println(user2.hashCode());
    }

}

